import type { Book, HoldRequest, Repository, User } from '@/payload-types'
import { getPayload, PaginatedDocs } from 'payload'
import config from '@/payload.config'
import { Card, CardContent, CardDescription, CardFooter, CardHeader, CardTitle } from '../ui/card'
import Image from 'next/image'
import { BorderTrail } from 'components/motion-primitives/border-trail'
import clsx from 'clsx'
import Link from 'next/link'
import { LoginForm } from '../login-form'

const stats = [
  { name: 'Outbound Loans', stat: '13' },
  { name: 'Active Holds', stat: '6' },
  { name: 'Hold Requests', stat: '3', shouldHighlight: true },
]

type Props = {
  user?: User
}
const UserFeed = async (props: Props) => {
  const { user } = props
  const isLoggedIn = !!user

  if (!isLoggedIn)
    return (
      <div className="flex w-full max-w-sm flex-col gap-6 mx-auto my-6">
        <LoginForm />
      </div>
    )

  const payloadConfig = await config
  const payload = await getPayload({ config: payloadConfig })

  const holdRequests = (await payload.find({
    collection: 'holdRequests',
    limit: 10,
    depth: 3,
    select: {
      copy: true,
      dateRequested: true,
      repository: true,
      book: true,
    },
    where: {
      userRequested: {
        equals: user?.id,
      },
      isCheckedOut: {
        not_equals: true,
      },
    },
  })) as PaginatedDocs<HoldRequest>

  return (
    <section>
      <div className="my-6">
        <h3 className="text-lg font-semibold text-foreground">Outbound Activity</h3>
        <dl className="mt-5 grid grid-cols-1 gap-5 sm:grid-cols-3">
          {stats.map((item) => (
            <div
              key={item.name}
              className={clsx(
                'relative overflow-hidden rounded-lg  accent-background px-4 py-5 shadow-sm sm:p-6',
                !item.shouldHighlight ? 'border-1 border-muted-foreground' : '',
              )}
            >
              {!!item.shouldHighlight && (
                <BorderTrail
                  style={{
                    boxShadow:
                      '0px 0px 60px 30px rgb(200 255 200 / 50%), 0 0 100px 60px rgb(0 0 0 / 50%), 0 0 140px 90px rgb(0 0 0 / 50%)',
                  }}
                  size={100}
                />
              )}
              <dt className="truncate text-sm font-medium text-muted-foreground">{item.name}</dt>
              <dd className="mt-1 text-3xl font-semibold tracking-tight text-foreground">
                {item.stat}
              </dd>
            </div>
          ))}
        </dl>
      </div>

      <div className="my-6">
        <h2 className="text-lg font-semibold text-foreground">Inbound Activity</h2>
        <div className="my-3">
          <h3 className="text-base font-semibold text-muted-foreground">Your Holds</h3>
          <ul className="flex flex-wrap justify gap-4 last-child-adjustment">
            {holdRequests.docs?.map((h) => {
              const book = h.book as Book
              const repository = h.repository as Repository

              const formatedDateRequested = h.dateRequested
                ? new Date(h.dateRequested).toDateString()
                : ''

              return (
                <li className="inline-block" key={book.isbn}>
                  <Link className="block hover:scale-105 transition-all" href={`/books/${book.id}`}>
                    <Card className="w-48 pt-4 pb-2">
                      <CardHeader className="-mb-4">
                        <CardTitle className="text-overflow-ellipsis line-clamp-2">
                          {book.title}
                        </CardTitle>
                        <CardDescription>{repository.abbreviation}</CardDescription>
                      </CardHeader>
                      <CardContent>
                        <Image
                          alt="book cover"
                          className="mx-auto w-full"
                          width={120}
                          height={180}
                          src={
                            book.isbn
                              ? `https://covers.openlibrary.org/b/isbn/${book.isbn}-M.jpg`
                              : '/images/book-48.svg'
                          }
                        />
                      </CardContent>
                      <CardFooter className="text-muted-foreground text-sm block -mt-5">
                        <span className="block w-full text-xs">Requested On</span>
                        <span className="block w-full">{formatedDateRequested}</span>
                      </CardFooter>
                    </Card>
                  </Link>
                </li>
              )
            })}
          </ul>
        </div>
      </div>
    </section>
  )
}

export default UserFeed
