'use client'

import { useForm } from 'react-hook-form'
import { z } from 'zod'
import { zodResolver } from '@hookform/resolvers/zod'

import { Button } from '@/components/ui/button'
import {
  Form,
  FormControl,
  FormField,
  FormItem,
  FormLabel,
  FormMessage,
} from '@/components/ui/form'
import { Input } from '@/components/ui/input'
import { useState } from 'react'
import { Loader2 } from 'lucide-react'
import { searchBooks } from '@/serverActions/SearchBooks'
import { PaginatedDocs } from 'payload'
import { Book } from '@/payload-types'
import { toast } from 'sonner'

const formSchema = z.object({
  title: z.string().max(80),
  author: z.string().max(80),
})

type Props = {
  initialTitle?: string
  initialAuthor?: string
  onSearchResult?: (results: PaginatedDocs<Book> | null) => void
}
const SearchBooksInlineForm = (props: Props) => {
  const { initialTitle, initialAuthor, onSearchResult } = props

  const form = useForm<z.infer<typeof formSchema>>({
    resolver: zodResolver(formSchema),
    defaultValues: {
      title: initialTitle || '',
      author: initialAuthor || '',
    },
  })

  const [isSearching, setIsSearching] = useState(false)

  const onSubmit = async (values: z.infer<typeof formSchema>) => {
    if (isSearching) return
    setIsSearching(true)

    try {
      const searchResults = await searchBooks(values)
      if (searchResults && onSearchResult) onSearchResult(searchResults)
    } catch (_) {
      toast('There was an issue with that search')
    } finally {
      setIsSearching(false)
    }
  }

  return (
    <Form {...form}>
      <form
        onSubmit={form.handleSubmit(onSubmit)}
        className="flex flex-wrap sm:flex-nowrap items-end gap-2"
      >
        <FormField
          control={form.control}
          name="title"
          render={({ field }) => (
            <FormItem className="sm:w-auto w-full block">
              <FormLabel>Title</FormLabel>
              <FormControl>
                <Input placeholder="Title" {...field} />
              </FormControl>
              <FormMessage />
            </FormItem>
          )}
        />
        <FormField
          control={form.control}
          name="author"
          render={({ field }) => (
            <FormItem className="sm:w-auto w-full block">
              <FormLabel>Author</FormLabel>
              <FormControl>
                <Input placeholder="Author" {...field} />
              </FormControl>
              <FormMessage />
            </FormItem>
          )}
        />
        <Button
          className="sm:w-auto w-full block cursor-pointer"
          disabled={isSearching}
          type="submit"
        >
          {isSearching ? <Loader2 className="animate-spin mx-auto" /> : <span>Search</span>}
        </Button>
      </form>
    </Form>
  )
}

export default SearchBooksInlineForm
